#!/usr/bin/env node
var currentBranch;
var Events = require('events').EventEmitter;
var ee = new Events();
var emit = ee.emit;
var exec = require('child_process').exec;
var on = ee.on;
var prompt = require('prompt');
var Q = require('q');

if(process.argv.length < 3) {
  console.error('Must provide an expression.');
  process.exit(1);
}

_initializeEvents();

emit('set-current-branch');

function _isError(err) {
  return (err && err.code === 1);
}

function _displayWarning() {
  console.log('==============================');
  console.log('        !!! Warning !!!       ');
  console.log('==============================');
}

function _initializeEvents() {
  on('are-you-sure', function areYouReallySure(branches) {
    var hasMaster = branches.indexOf('master') !== -1;
    var hasDevelop = branches.indexOf('develop') !== -1;
    var msg;
    var warningBranches = [];

    if(hasMaster) { warningBranches.push('master'); }
    if(hasDevelop) { warningBranches.push('develop'); }

    _displayWarning();
    msg = 'Are you really sure that you want to delete the following branches: ';

    if(warningBranches.length > 0) { msg += warningBranches.join(', '); }

    msg += ' ?';

    console.log(msg);

    prompt.start();

    prompt.get({
      properties: [{
        name: 'yesOrNo',
        description: '(y/n)',
        pattern: /^(y|n)$/,
        message: 'Only y or n are valid answers',
        required: true
      }]
    }, function promptCallback(err, results) {
      if(results[0] === 'y') {
        emit('remove-branches', branches);
      } else {
        emit('cancel-remove-branches');
      }
    });
  });

  on('ask-to-proceed', function askToProceedEvent(branches) {
    var msg;
    var potentialMistake = false;

    branches.map(function mapBranches(branch) {
      msg = '- ' + branch;

      if(branch === 'master' || branch === 'develop') {
        msg += ' <-- WARNING ** POTENTIAL MISTAKE **';
        potentialMistake = true;
      }

      console.log(msg);
    });

    console.log('==============================');
    console.log('Would you like to delete the selected branches?');

    prompt.start();

    prompt.get({
      properties: [{
        name: 'yesOrNo',
        description: '(y/n)',
        pattern: /^(y|n)$/,
        message: 'Only y or n are valid answers',
        required: true
      }]
    }, function proceedCallback(err, results) {
      if(results[0] === 'y') {
        if(potentialMistake) {
          emit('are-you-sure', branches);
        } else {
          emit('remove-branches', branches);
        }
      } else {
        emit('cancel-remove-branches');
      }
    });
  });

  on('cancel-remove-branches', function canceledRemoveEvent() {
    console.warn('Aborting...');
    process.exit(0);
  });

  on('error', function error() {
    console.error('Exited unexpectedly...');
    process.exit(1);
  });

  on('get-buffer', function getBufferEvent() {
    exec('git branch', function bufferCallback(err, buffer, stderr) {
      if(_isError(err)) {
        emit('get-buffer:error', err);
      } else {
        emit('parse-buffer', buffer);
      }
    });
  });

  on('get-buffer:error', function getBufferError(e) {
    console.error('Unable to retrieve response buffer');
    console.error(e.message);
  });

  on('local-branch-removed:success', function removeLocalSuccessEvent(branch) {
    console.log('Local branch removed: ' + branch);
  });

  on('local-branch-removed:error', function removeLocalErrorEvent(e, branch) {
    console.log('Failed to remove local branch: ' + branch+ '\n');
  });

  on('nothing-matches:error', function nothingMatchesError() {
    console.log('Nothing seems to match the provided expression.');
  });

  on('parse-buffer', function parseBufferEvent(buffer) {
    var currentBranchIndex;
    var exp;
    var i;
    var branches;

    try {
      exp = new RegExp(process.argv[2]);
      branches = buffer.toString();

      branches = branches.replace(/\n/g, ',');
      branches = branches.substr(0, branches.lastIndexOf(','))
      .replace(/[*] /, '')
      .replace(/\s\s/g, '')
      .split(',');

      currentBranchIndex = branches.indexOf(currentBranch);
      branches.splice(currentBranchIndex, 1);

      for(i = 0; i < branches.length; i++) {
        exp.lastIndex = 0;

        if(!exp.test(branches[i])) {
          branches.splice(i, 1);
          i--;
        }
      }

      if(branches.length < 1) {
        emit('nothing-matches:error');
      } else {
        emit('ask-to-proceed', branches);
      }
    } catch(e) {
      emit('parse-buffer:error', e);
    }
  });

  on('parse-buffer:error', function parseBufferError(e) {
    console.error('Unable to parse results... exiting.');
    console.error(e.message);
    process.exit(1);
  });

  on('remove-branches', function removeBranchesEvent(branches) {
    branches.map(function mapBranches(branch) {
      _removeLocalBranch(branch);
      _removeRemoteBranch(branch);
    });

    function _removeLocalBranch(branch) {
      exec('git branch -D ' + branch, function removeRemoveCallback(err, out) {
        if(_isError(err)) {
          emit('local-branch-removed:error', err, branch);
        } else {
          emit('local-branch-removed:success', branch);
        }
      });
    }

    function _removeRemoteBranch(branch) {
      exec('git push origin :' + branch, function removeRemoveCallback(err, out) {
        if(_isError(err)) {
          emit('remote-branch-removed:error', err, branch);
        } else {
          emit('remote-branch-removed:success', branch);
        }
      });
    }
  });

  on('remote-branch-removed:success', function removeRemoteSuccessEvent(branch) {
    console.log('Remote branch removed: ' + branch);
  });

  on('remote-branch-removed:error', function removeRemoteSuccessError(e, branch) {
    console.log('Failed to remove remote branch: ' + branch + '\n');
  });

  on('set-current-branch', function setCurrentBranch() {
    exec('git rev-parse --abbrev-ref HEAD', function getCurrentBranch(err, out) {
      if(_isError(err)) {
        emit('set-current-branch:error', err);
      } else {
        currentBranch = out.replace(/\n/g, '');

        emit('get-buffer');
      }
    });
  });

  on('set-current-branch:error', function setCurrentBranchError() {
    console.error('Unable to set current branch... exiting.');
    process.exit(1);
  });
}
